{
  Author      : Nick Protopapas
  Created On  : 2000/09/01

  Description : VCL component wrapper for Embedded Async Pluggable Protocol

  Resources   : eappProtocolHandler by Christopher D. Coppola
                     1999, Systems & Computer Technology Corporation

  The TIdMimeTable object is taked from the Indy Project ( www.jedi-project.com )
}

unit VirtualSite;

interface

uses Classes, Windows, SysUtils, ActiveX, UrlMon, Dialogs, axctrls, ComObj,
     Controls, OleCtrls, StdVCL, SHDocVw, DsgnIntf, Forms;

const
  Class_TeapProtocol: TGUID = '{5902C4EB-8397-4FF6-B5D8-741A51D26543}';
  CLSID_EAPDataMoniker : TGUID = '{1339DAF7-BF1D-4DF3-9625-D928A5EC44AA}';
  PROTOCOL_SCHEME = 'eapp';
  PROTOCOL_CLASS_NAME = 'EAPProtocol';
  PROTOCOL_DESC = 'Embedded Async Pluggable Protocol';

type

{ TVirtualSite }

  TServeMethod = Procedure ( FileName, QueryString: string; var Return_MIME_Type: string; var Content: TMemoryStream) of object;

  TVirtualSite = class;

  TFileStringProperty = class(TStringProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  TFileString = type String;

  TWebSiteFile = class(TCollectionItem)
  private
    FOnServe: TServeMethod;
    FVirtualName: string;
    FFileName: TFileString;
    FData: Pointer;
    FDataSize: Integer;
    procedure WriteData(Stream: TStream);
    procedure ReadData(Stream: TStream);
    procedure SetFileName(const Value: TFileString);
  protected
    procedure DefineProperties(Filer: TFiler); override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
    function Empty: Boolean;
    procedure LoadFromFile(const FileName: String);
    procedure LoadFromStream(S: TStream);
    procedure SaveToFile(const FileName: String);
    procedure SaveToStream(S: TStream);
    procedure GetStream(var S: TMemoryStream);
    procedure SetHiddenFileName(Value: String);
  published
    property FileName: TFileString read FFileName write SetFileName;
    property OnServe: TServeMethod read FOnServe write FOnServe;
    property VirtualName: string read FVirtualName write FVirtualName;
  end;

  TWebSiteFiles = Class(TCollection)
  private
    FVirtualSite: TVirtualSite;
    function GetItem(Index: Integer): TWebSiteFile;
    procedure SetItem(index: Integer; Value: TWebSiteFile);
  protected
    procedure Update(Item: TCollectionItem); override;
    function GetOwner: TPersistent; override;
  public
    constructor Create(VirtualSite: TVirtualSite);
    function Add: TWebSiteFile;
    property Items[Index: Integer]: TWebSiteFile read GetItem write SetItem; default;
  end;

  TVirtualSite = class(TComponent)
  private
    FWebSiteFiles: TWebSiteFiles;
  protected
    procedure SetWebSiteFiles(const Value: TWebSiteFiles);
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
  published
    property WebSiteFiles: TWebSiteFiles read FWebSiteFIles write SetWebSiteFiles;
  end;

  TeapProtocol = class(TComObject, IInternetProtocol, IInternetProtocolInfo)
  private
    fIProtSink : IInternetProtocolSink;
    fIBindInfo : IInternetBindInfo;
    fSinkCount,
    fBindCount,
    fStrmCount : integer;
    fBindType : Cardinal;
    fBindInfo : TBindInfo;
    fGrfSTI : DWORD;
    fLocked : Boolean;
    fDone,
    fReallyDone : Boolean;
    fcbTotalReadbyURLMon : DWORD;
    fcbTotalRead : DWORD;
    fcbTotalWritten : DWORD;
    fcbTotalSize : DWORD;
    fStrmRead : IStream;
    function DoBind : HResult;
    function PrepareSecurityUrl(szUrl : string): string;
  protected
    function DoParse(szUrl : string) : HResult; virtual; abstract;
    function DoBuildContent(var iWritten : longint): HResult; virtual; abstract;
    { IInternetProtocol }
    function Start(szUrl: PWideChar; OIProtSink: IInternetProtocolSink;
      OIBindInfo: IInternetBindInfo; grfPI, dwReserved: DWORD): HResult; stdcall;
    function Continue(const ProtocolData: TProtocolData): HResult; stdcall;
    function Abort (hrReason: HResult; dwOptions: DWORD): HResult; stdcall;
    function Terminate(dwOptions: DWORD): HResult; stdcall;
    function Suspend: HResult; stdcall;
    function Resume: HResult; stdcall;
    function Read(pv: Pointer; cb: ULONG; out cbRead: ULONG): HResult; stdcall;
    function Seek(dlibMove: LARGE_INTEGER; dwOrigin: DWORD; out libNewPosition: ULARGE_INTEGER): HResult; stdcall;
    function LockRequest (dwOptions: DWORD): HResult; stdcall;
    function UnlockRequest: HResult; stdcall;
    property StrmRead : IStream read fStrmRead;
    { IInternetProtocolInfo }
    function ParseUrl(pwzUrl: LPCWSTR; ParseAction: TParseAction; dwParseFlags: DWORD;
      pwzResult: LPWSTR; cchResult: DWORD; pcchResult: DWORD;
      dwReserved: DWORD): HResult; stdcall;
    function CombineUrl(pwzBaseUrl, pwzRelativeUrl: PWideChar; dwCombineFlags: DWORD;
      pwzResult: PWideChar; cchResult: DWORD; out pcchResult: DWORD;
      dwReserved: DWORD): HResult; stdcall;
    function CompareUrl(pwzUrl1, pwzUrl2: PWideChar; dwCompareFlags: DWORD): HResult; stdcall;
    function QueryInfo(pwzUrl: PWideChar; QueryOption: TQueryOption; dwQueryFlags: DWORD;
      pBuffer: Pointer; cbBuffer: DWORD; var cbBuf: DWORD; dwReserved: DWORD): HResult; stdcall;
  end;

  TeapFileProtocol = class(TeapProtocol)
  private
    fFilename : string;
    fQueryStrings: string;
  protected
    function DoParse(szUrl : string) : HResult; override;
    function DoBuildContent(var iWritten : longint): HResult; override;
  end;

  TIdMimeTable = Class
  protected
    FMIMEList: TStringList;
    FFileExt: TStringList;
  public
    procedure BuildCache; virtual;
    function GetFileMIMEType(const fileName: String): String;
    function getDefaultFileExt(Const MIMEType: string): String;
    constructor Create(Autofill: boolean=true); virtual;
    destructor Destroy; override;
  end;

var
  VS: TVirtualSite;

implementation

uses
  WinInet, Registry;

var
  MIME_TYPE: string;

{ TFileStringProperty }

procedure TFileStringProperty.Edit;
begin
  with TOpenDialog.Create(nil) do
  try
    Filter:='All Files|*.*';
    DefaultExt:='*.*';
    FileName:=GetStrValue;
    if Execute then
      SetStrValue(FileName);
    finally
      Free;
    end;
end;

function GetMIMETypeFromFile(AFile: TFileName): string;
var
  MIMEMap: TIdMIMETable;
begin
  MIMEMap := TIdMimeTable.Create(true);
  try
    result := MIMEMap.GetFileMIMEType(AFile);
  finally
    MIMEMap.Free;
  end;
end;

function TFileStringProperty.GetAttributes: TPropertyAttributes;
begin
  Result:=[paDialog];
end;

function inetCanonUrl(var szUrl : string; Flags : integer): Boolean;
var
  dwUrlSize : DWORD;
  szCanonUrl : array[0..INTERNET_MAX_PATH_LENGTH] of char;
begin
  result := False;
  dwUrlSize := INTERNET_MAX_PATH_LENGTH;
  if InternetCanonicalizeUrl(PChar(szUrl), szCanonUrl, dwUrlSize, Flags) then begin
    szUrl := Copy(szCanonUrl, 1, dwUrlSize);
    result := True;
  end;
end;

function TeapFileProtocol.DoBuildContent(var iWritten: Integer): HResult;
var
  OS: TOLEStream;
  MS : TMemoryStream;
  I: Integer;
  ServeMethod: TServeMethod;
  Return_MIME_Type: string;
begin
  iWritten := 0;
  result := S_FALSE;
  MS:= TMemoryStream.Create;

  for I:=0 to VS.WebSiteFiles.Count-1 do
  begin
    if LowerCase(VS.WebSiteFiles.Items[I].VirtualName)=LowerCase(fFileName) then
    begin
      ServeMethod:=VS.WebSiteFiles.Items[I].OnServe;
      if assigned(ServeMethod) then ServeMethod( fFileName, fQueryStrings, Return_MIME_TYPE, MS) else
      begin
        if not VS.WebSiteFiles.Items[I].Empty then
        begin
          VS.WebSiteFiles.Items[I].SaveToStream(MS);
          Return_MIME_TYPE:=GetMIMETypeFromFile(VS.WebSiteFiles.Items[I].VirtualName);
        end else
        begin
          MS.Write('Object has empty size ...',25);
          Return_MIME_TYPE:=CFSTR_MIME_HTML;
        end;
      end;
      break;
    end;
  end;
  if MS.Size=0 then
  begin
    MS.Write('Object could not be found ...',29);
    Return_MIME_TYPE:=CFSTR_MIME_HTML;
  end;

  MIME_TYPE:=Return_MIME_TYPE;
  try
    MS.Position:=0;
    OS:=TOLEStream.Create( StrmRead );
    try
      OS.CopyFrom(MS, MS.Size);
      iWritten := OS.Size;
      result := S_OK;
    finally
      OS.Free;
    end;
  finally
    MS.Free;
  end;
end;

function TeapFileProtocol.DoParse(szUrl: string): HResult;
var
  szProtStr : string;
begin
  result := S_OK;
  szProtStr := szUrl;
  inetCanonUrl(szProtStr, ICU_DECODE or ICU_NO_ENCODE);
  Delete(szProtStr, 1, Length(PROTOCOL_SCHEME)+1);
  if ( Copy(szProtStr, 1, 2) = '//' ) then
    Delete(szProtStr, 1, 2);
  if copy(szProtStr,Length(szProtStr),1)='/' then
  begin
    Delete(szProtStr,length(szProtStr),1);
  end;
  fFileName:=szProtStr;
  if pos('?',fFileName)<>0 then
  begin
    fFileName:=Copy(szProtStr,1,pos('?',fFileName)-1);
    fQueryStrings:=Copy(szProtStr,pos('?',szProtStr)+1,Length(szProtStr)-pos('?',szProtStr));
  end else
  begin
    fQueryStrings:='';
  end;
end;

{ TVirtualSite }

constructor TVirtualSite.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  FWebSiteFiles:=TWebSiteFiles.Create(Self);
end;

destructor TVirtualSite.Destroy;
begin
  FWebSiteFIles.Free;
  inherited Destroy;
end;

{ IInternetProtocol }

function TeapProtocol.LockRequest (dwOptions: DWORD): HResult; stdcall;
begin
  fLocked := True;
  result := S_OK;
end;

function TeapProtocol.Read (pv: Pointer; cb: ULONG; out cbRead: ULONG): HResult;
var
  hr : HResult;
  s: string;      //for debug only
begin
  hr := S_OK;
  result := S_OK;
  if not fReallyDone then
    hr := fStrmRead.Read(pv, cb, @cbRead);
  s := string(PChar(pv));         //for debug only
  inc(fcbTotalReadbyURLMon, cbRead);
  if ( hr = S_OK ) and ( fcbTotalReadByURLMon = fcbTotalRead ) then
    if fDone then begin
      result := S_FALSE;
      fReallyDone := True;
    end
    else
      result := HResult(E_PENDING);
end;

function TeapProtocol.Seek(dlibMove: LARGE_INTEGER; dwOrigin: DWORD; out libNewPosition: ULARGE_INTEGER): HResult;
begin
  result := E_NOTIMPL;
end;

function TeapProtocol.UnlockRequest: HResult;
begin
  fLocked := False;
  result := S_OK;
end;

{ IInternetProtocolRoot }

function TeapProtocol.Abort (hrReason: HResult; dwOptions: DWORD): HResult; stdcall;
begin
  if (SUCCEEDED(hrReason)) then // Possibly Abort could get called with 0?
    hrReason := E_ABORT;
  // Notify Sink of abort
  if fIProtSink <> nil then
    fIProtSink.ReportResult(hrReason, 0, nil);
  result := S_OK;
end;

function TeapProtocol.Continue (const ProtocolData: TProtocolData): HResult;
begin
  result := E_FAIL;
  if ( ProtocolData.dwState = 1 ) then
    result := DoBind;
end;

function TeapProtocol.Resume: HResult; stdcall;
begin
  { Not currently Implemented }
  result := E_NOTIMPL;
end;

function TeapProtocol.Start( szUrl: PWideChar; OIProtSink: IInternetProtocolSink;
  OIBindInfo: IInternetBindInfo; grfPI, dwReserved: DWORD ): HResult; stdcall;
var
  protdata : TProtocolData;
  hr: HResult;
begin
  result := S_FALSE;
  fGrfSTI := grfPI;
  // Initialize variables for new download
  fcbTotalRead := 0;
  fcbTotalWritten := 0;
  fcbTotalSize := 0;
  fcbTotalReadByURLMON := 0;
  fDone := False;
  fLocked := False;
  fIProtSink := OIProtSink;
  fSinkCount := fIProtSink._AddRef;
  fIBindInfo := OIBindInfo;
  fBindCount := fIBindInfo._AddRef;
  fBindInfo.cbSize := sizeof(fBindInfo);
  fIBindInfo.GetBindInfo(fBindType, fBindInfo);
  hr := DoParse(szUrl);
  if (fGrfSTI and PI_PARSE_URL) <> 0 then begin
    if SUCCEEDED(hr) then
      result := S_OK
    else
      result := S_FALSE;
  end;
  if ( (fBindType and BINDF_NEEDFILE) <> 0 ) then begin

  end;
  if SUCCEEDED(hr) then begin
    { We successfully parsed the url, continue }
    if (fGrfSTI and PI_FORCE_ASYNC) <> 0 then begin
      hr := DoBind;
      result := hr;
    end else begin
      result := HResult(E_PENDING);
      protdata.grfFlags := PI_FORCE_ASYNC;
      protdata.dwState := 1;
      protdata.pData := nil;
      protdata.cbData := 0;
      if fIProtSink <> nil then
        fIProtSink.Switch(protdata)
      else
        result := E_INVALIDARG;
    end;
  end;
end;

function TeapProtocol.Suspend: HResult; stdcall;
begin
  { Not currently Implemented }
  result := E_NOTIMPL;
end;

function TeapProtocol.Terminate(dwOptions: DWORD): HResult; stdcall;
begin
  {
    Called by UrlMon.dll when we call pOIProtSink.ReportResult. All resources
    should be freed when this method is complete.
  }
  result := S_OK;
  if fStrmRead <> nil then begin
    fStrmCount := fStrmRead._Release;
    fStrmRead := nil;
  end;
  if fIProtSink <> nil then begin
    fSinkCount := fIProtSink._Release;
    fIProtSink := nil;
  end;
  if fIBindInfo <> nil then begin
    fBindCount := fIBindInfo._Release;
    fIBindInfo := nil;
  end;
  ReleaseBindInfo(fBindInfo);
end;

{ IInternetProtocolInfo }

function TeapProtocol.CombineUrl(pwzBaseUrl, pwzRelativeUrl: PWideChar;
  dwCombineFlags: DWORD; pwzResult: PWideChar; cchResult: DWORD;
  out pcchResult: DWORD; dwReserved: DWORD): HResult;
begin
  result := INET_E_DEFAULT_ACTION;
end;

function TeapProtocol.CompareUrl(pwzUrl1, pwzUrl2: PWideChar;
  dwCompareFlags: DWORD): HResult;
begin
 if ( CompareText(pwzUrl1, pwzUrl2) = 0 ) then
  result := S_OK
 else
  result := S_FALSE;
end;

function TeapProtocol.ParseUrl(pwzUrl: LPCWSTR; ParseAction: TParseAction; dwParseFlags: DWORD;
      pwzResult: LPWSTR; cchResult: DWORD; pcchResult: DWORD;
      dwReserved: DWORD): HResult; stdcall;
var
  sResult : string;
begin
  sResult := '';
  result := INET_E_DEFAULT_ACTION;
  case ParseAction of
    PARSE_CANONICALIZE : sResult := '';
    PARSE_FRIENDLY : sResult := 'Looking something up....';
    PARSE_SECURITY_URL : sResult := PrepareSecurityUrl(pwzUrl);
    PARSE_ROOTDOCUMENT : sResult := '';
    PARSE_DOCUMENT : sResult := '';
    PARSE_ANCHOR : sResult := '';
    PARSE_ENCODE : sResult := '';
    PARSE_DECODE : sResult := '';
    PARSE_PATH_FROM_URL : sResult := '';
    PARSE_URL_FROM_PATH : sResult := '';
    PARSE_MIME : sResult := CFSTR_MIME_HTML;
    PARSE_SERVER : sResult := '';
    PARSE_SCHEMA : sResult := '';
    PARSE_SITE : sResult := '';
    PARSE_DOMAIN : sResult := '';
    PARSE_LOCATION : sResult := '';
    PARSE_SECURITY_DOMAIN : sResult := '';
//    PARSE_ESCAPE : sResult := '';
//    PARSE_UNESCAPE : sResult := '';
  else
    result := INET_E_DEFAULT_ACTION;
  end; //case

  if sResult <> '' then begin
    pcchResult := Length(sResult)*2;
    if cchResult >= pcchResult then begin
      StringToWideChar(sResult, pwzResult, pcchResult);
      result := S_OK;
    end else
      result := S_FALSE;
  end;
end;

function TeapProtocol.QueryInfo(pwzUrl: PWideChar;
  QueryOption: TQueryOption; dwQueryFlags: DWORD; pBuffer: Pointer;
  cbBuffer: DWORD; var cbBuf: DWORD; dwReserved: DWORD): HResult;
var
  sResult : string;
begin
  sResult := 'true';
  result := INET_E_DEFAULT_ACTION;
  case QueryOption of
    QUERY_EXPIRATION_DATE     : sResult := '';
    QUERY_TIME_OF_LAST_CHANGE : sResult := '';
    QUERY_CONTENT_ENCODING    : sResult := '';
    QUERY_CONTENT_TYPE        : sResult := '';
    QUERY_REFRESH             : sResult := '';
    QUERY_RECOMBINE           : sResult := '';
    QUERY_CAN_NAVIGATE        : sResult := '';
    QUERY_USES_NETWORK        : sResult := '';
    QUERY_IS_CACHED           : sResult := '';
    QUERY_IS_INSTALLEDENTRY   : sResult := '';
    QUERY_IS_CACHED_OR_MAPPED : sResult := '';
    QUERY_USES_CACHE          : sResult := '';
  end; //case
  if sResult <> '' then begin
    cbBuf := Length(sResult)*2;
    if cbBuffer >= cbBuf then begin
      StringToWideChar(sResult, pBuffer, cbBuf);
      result := S_OK;
    end else
      result := S_FALSE;
  end;
end;

// Misc Helper Routines
function TeapProtocol.DoBind: HResult;
var
  hr : HResult;
  iWritten : longint;
  iStmPos : Int64;
  wcStatus : array[0..40] of WideChar;
  DestSize: Integer;
begin
  hr := CreateStreamOnHGlobal(0, True, fStrmRead);
  if SUCCEEDED(hr) then begin
    fStrmCount := fStrmRead._AddRef;
    DoBuildContent(iWritten);
    fcbTotalRead := iWritten;
    if fcbTotalRead = 0 then begin
      Abort(0, 0);
      result := S_OK;
    end else begin
      fStrmRead.Seek(0, soFromBeginning, iStmPos);

      DestSize := Length(MIME_TYPE)+1;
      StringToWideChar(MIME_TYPE, wcStatus, DestSize);

      fIProtSink.ReportProgress( BINDSTATUS_VERIFIEDMIMETYPEAVAILABLE,
        wcStatus);
      fDone := True;
      fIProtSink.ReportData(
        BSCF_LASTDATANOTIFICATION or BSCF_DATAFULLYAVAILABLE,
        fcbTotalRead, fcbTotalRead);
      fIProtSink.ReportResult(S_OK, S_OK, nil);
      result := S_OK;
    end;
  end else
    result := hr;
end;

function TeapProtocol.PrepareSecurityUrl(szUrl: string): string;
begin
  result := 'file:c|\';
end;

procedure TVirtualSite.SetWebSiteFiles(const Value: TWebSiteFiles);
begin
  FWebSiteFiles.Assign(Value);
end;

{ TWebSiteFile }

procedure TWebSiteFile.GetStream(var S: TMemoryStream);
begin
  if not Empty then
    S.Write(FData^,FDataSize);
end;

procedure TWebSiteFile.SetHiddenFileName(Value: String);
begin
  FFileName:=Value;
end;

procedure TWebSiteFile.Assign(Source: TPersistent);
var
  S: TMemoryStream;
begin
  if source is TWebSiteFile then
  begin
    S:=TMemoryStream.Create;
    FFileName:=(TWebSiteFile(Source).FileName);
    TWebSiteFile(Source).GetStream(S);
    LoadFromStream(S);
  end;
end;

procedure TWebSiteFile.SaveToStream(S: TStream);
begin
  if not Empty then
    S.Write(FData^,FDataSize);
end;

procedure TWebSiteFile.SetFileName(const Value: TFileString);
begin
  if value<>'' then
  begin
    FFileName:=ExtractFileName(Value);
    if (not (csLoading in TWebSiteFiles(Collection).FVirtualSite.ComponentState)) and FileExists(Value) then
    begin
      LoadFromFile(Value);
      if FVirtualName='' then FVirtualName:=FFileName;
    end;
  end else
  begin
    FFileName:='';
    if not Empty then
      FreeMem(FData, FDataSize);
      FDataSize:=0;
    end;
end;

procedure TWebSiteFile.WriteData(Stream: TStream);
begin
  SaveToStream(Stream);
end;

procedure TWebSiteFile.LoadFromFile(const FileName: String);
var
  F: TFileStream;
begin
  F:=TFileStream.Create(FileName, fmOpenRead);
  try
    LoadFromStream(F);
  finally
    F.Free;
  end;
end;

procedure TWebSiteFile.LoadFromStream(S: TStream);
begin
  if not empty then
    FreeMem(FData, FDataSize);
  FDataSize:=0;
  FData:=AllocMem(S.Size);
  FDataSize:=S.Size;
  S.Read(FData^,FDataSize);
end;

procedure TWebSiteFile.ReadData(Stream: TStream);
begin
  LoadFromStream(Stream);
end;

procedure TWebSiteFile.DefineProperties(Filer: TFiler);
  function DoWrite: Boolean;
  begin
    if Filer.Ancestor <> nil then
      Result:= not (Filer.Ancestor is TWebSiteFile) //or
//        not Equal(TWebSiteFile(Filer.Ancestor))
    else
      Result:= not Empty;
  end;

begin
  inherited DefineProperties(Filer);
  Filer.DefineBinaryProperty('Data', ReadData, WriteData, DoWrite);
end;

procedure TWebSiteFile.SaveToFile(const FileName: String);
var
  F: TFIleSTream;
begin
  F:=TFileStream.Create(FileName, fmCreate);
  try
    SaveToStream(F);
  finally
    F.Free;
  end;
end;

function TWebSiteFile.Empty: Boolean;
begin
  Result:=FDataSize = 0;
end;

constructor TWebSiteFile.Create(Collection: TCollection);
begin
  inherited Create(Collection);
  // DO Here
end;

destructor TWebSiteFile.Destroy;
begin
  if not Empty then
    FreeMem(FData, FDataSize);
  inherited Destroy;
end;

{ TWebSiteFiles }

function TWebSiteFiles.Add: TWebSiteFile;
begin
  Result:=TWebSiteFile(inherited Add);
end;

constructor TWebSiteFiles.Create(VirtualSite: TVirtualSite);
begin
  inherited Create(TWebSiteFile);
  FVirtualSite:= VirtualSite;
end;

function TWebSiteFiles.GetItem(Index: Integer): TWebSiteFile;
begin
  Result:=TWebSiteFile(inherited GetItem(Index));
end;

function TWebSiteFiles.GetOwner: TPersistent;
begin
  Result := FVirtualSite;
end;

procedure TWebSiteFiles.SetItem(index: Integer; Value: TWebSiteFile);
begin
  inherited SetItem(Index, Value);
end;

procedure TWebSiteFiles.Update(Item: TCollectionItem);
begin
  inherited;
end;

  {$IFNDEF VCL5ORABOVE}
  function CreateTRegistry: TRegistry;
  begin
    Result := TRegistry.Create;
  end;
  {$ELSE}
  function CreateTRegistry: TRegistry;
  begin
    Result := TRegistry.Create(KEY_READ);
  end;
{$ENDIF}

procedure TIdMimeTable.BuildCache;
var
  reg: TRegistry;
  KeyList: TStringList;
  i: Integer;
begin
  // Build the file type/MIME type map
  Reg := CreateTRegistry; try
    KeyList := TStringList.create;
    try
      Reg.RootKey := HKEY_CLASSES_ROOT;
      Reg.OpenKeyReadOnly('\');
      Reg.GetKeyNames(KeyList);
      reg.Closekey;
      // get a list of registered extentions
      for i := 0 to KeyList.Count - 1 do
      begin
        if Copy(KeyList[i], 1, 1) = '.' then
        begin
          reg.OpenKeyReadOnly(KeyList[i]);
          if Reg.ValueExists('Content Type') then
          begin
            FFileExt.Values[KeyList[i]] := Reg.ReadString('Content Type');
          end;
          reg.CloseKey;
        end;
      end;
      Reg.OpenKeyreadOnly('\MIME\Database\Content Type');

      // get a list of registered MIME types
      KeyList.Clear;

      Reg.GetKeyNames(KeyList);
      reg.Closekey;
      for i := 0 to KeyList.Count - 1 do
      begin
        Reg.OpenKeyreadOnly('\MIME\Database\Content Type\' + KeyList[i]);
        FMIMEList.Values[reg.ReadString('Extension')] := KeyList[i];
        Reg.CloseKey;
      end;
    finally
      KeyList.Free;
    end;
  finally
    reg.free;
  end;
end;

constructor TIdMimeTable.Create(Autofill: boolean);
begin
  FFileExt := TStringList.Create;
  FMIMEList := TStringList.Create;
  if Autofill then
    BuildCache;
end;

destructor TIdMimeTable.Destroy;
begin
  FreeAndNil(FMIMEList);
  FreeAndNil(FFileExt);
  inherited;
end;

function TIdMimeTable.getDefaultFileExt(const MIMEType: string): String;
begin
  result := FMIMEList.Values[MIMEType];
  if Length(result) = 0 then
  begin
    BuildCache;
    result := FMIMEList.Values[MIMEType];;
  end;
end;

function TIdMimeTable.GetFileMIMEType(const fileName: String): String;
begin
  result := FFileExt.Values[ExtractFileExt(FileName)];
  if Length(result) = 0 then
  begin
    BuildCache;
    result := FMIMEList.Values[ExtractFileExt(FileName)];
    if Length(result) = 0 then
    begin
      result := 'application/octet-stream';{do not localize}
    end;
  end;
end;


end.
