(* aaltContentBrowser ----------------------------------------------------------

  Learning Suite
  Common
  Author      : Christopher D. Coppola
  Updated By  : cdc - 1998/11/20
                cdc - 19990917

  Description: This unit implements the TeappWebBrowser. TeappWebBrowser
               is the heart Learning Suite's Course Explorer. It is the HTML
               browser component that renders content and allows interaction
               with content.

   1999, Systems & Computer Technology Corporation

------------------------------------------------------------------------------*)

unit eappBrowser;

interface

uses
  Windows, ActiveX, Classes, Graphics, Controls, OleCtrls, StdVCL,
  SHDocVw, UrlMon, VirtualSite, DsgnIntf;

type

  TAPPWebBrowser = class(TWebBrowser)
  private
    fInternetSession : IInternetSession;
    fProtocolFactory : IClassFactory;
    fProtocolClass : TGUID;
    FVirtualSite: TVirtualSite;
    procedure ReleaseProtocol;
    procedure SetVirtualSite(const Value: TVirtualSite);
  protected
  public
    destructor Destroy; override;
    constructor Create( AOwner: TComponent); override;
    function InitProtocol : Boolean;
    property ProtocolClass : TGUID read fProtocolClass write fProtocolClass;
  published
    property VirtualSite: TVirtualSite read FVirtualSite write SetVirtualSite;
  end;

procedure Register;

implementation

uses
  SysUtils, MSHTML, WinInet, ComObj;

procedure Register;
begin
  RegisterComponents('Samples', [TAPPWebBrowser]);
  RegisterComponents('Samples', [TVirtualSite]);
  RegisterPropertyEditor(TypeInfo(TFileString), TWebSiteFile, '', TFileStringProperty);
end;

constructor TAPPWebBrowser.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  ProtocolClass:=Class_TeapProtocol;
  if not (csDesigning in ComponentState) then InitProtocol;
end;

destructor TAPPWebBrowser.Destroy;
begin
  ReleaseProtocol;
  inherited Destroy;
end;

function TAPPWebBrowser.InitProtocol: Boolean;
begin
  result := False;
  try
    CoGetClassObject(fProtocolClass, CLSCTX_SERVER, nil, IClassFactory, fProtocolFactory);
    fProtocolFactory._AddRef;
    CoInternetGetSession(0, fInternetSession, 0);
    if fInternetSession <> nil then begin
      fInternetSession._AddRef;
      result := ( fInternetSession.RegisterNameSpace(fProtocolFactory, fProtocolClass,
        PROTOCOL_SCHEME, 0, nil, 0) = S_OK );
    end;
  except
    result := False;
  end;
end;

procedure TAPPWebBrowser.ReleaseProtocol;
begin
  if ( fInternetSession <> nil ) then begin
    fInternetSession.UnregisterNameSpace(fProtocolFactory, PROTOCOL_SCHEME);
    fInternetSession._Release;
  end;
  if ( fProtocolFactory <> nil ) then
    fProtocolFactory._Release;
end;

procedure TAPPWebBrowser.SetVirtualSite(const Value: TVirtualSite);
begin
  FVirtualSite := Value;
  VS := Value;
end;

end.
